@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -28,29 +28,36 @@ This module contains an :class:`AbstractDataFileDialog`.
 
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "05/03/2018"
+__date__ = "05/03/2019"
 
 
 import sys
 import os
 import logging
-import numpy
 import functools
+from distutils.version import LooseVersion
+
+import numpy
+import six
+
 import silx.io.url
 from silx.gui import qt
 from silx.gui.hdf5.Hdf5TreeModel import Hdf5TreeModel
 from . import utils
-from silx.third_party import six
 from .FileTypeComboBox import FileTypeComboBox
-try:
-    import fabio
-except ImportError:
-    fabio = None
+
+import fabio
 
 
 _logger = logging.getLogger(__name__)
 
 
+DEFAULT_SIDEBAR_URL = True
+"""Set it to false to disable initilializing of the sidebar urls with the
+default Qt list. This could allow to disable a behaviour known to segfault on
+some version of PyQt."""
+
+
 class _IconProvider(object):
 
     FileDialogToParentDir = qt.QStyle.SP_CustomBase + 1
@@ -143,14 +150,22 @@ class _SideBar(qt.QListView):
         :rtype: List[str]
         """
         urls = []
-        if qt.qVersion().startswith("5.") and sys.platform in ["linux", "linux2"]:
+        version = LooseVersion(qt.qVersion())
+        feed_sidebar = True
+
+        if not DEFAULT_SIDEBAR_URL:
+            _logger.debug("Skip default sidebar URLs (from setted variable)")
+            feed_sidebar = False
+        elif version.version[0] == 4 and sys.platform in ["win32"]:
+            # Avoid locking the GUI 5min in case of use of network driver
+            _logger.debug("Skip default sidebar URLs (avoid lock when using network drivers)")
+            feed_sidebar = False
+        elif version < LooseVersion("5.11.2") and qt.BINDING == "PyQt5" and sys.platform in ["linux", "linux2"]:
             # Avoid segfault on PyQt5 + gtk
             _logger.debug("Skip default sidebar URLs (avoid PyQt5 segfault)")
-            pass
-        elif qt.qVersion().startswith("4.") and sys.platform in ["win32"]:
-            # Avoid 5min of locked GUI relative to network driver
-            _logger.debug("Skip default sidebar URLs (avoid lock when using network drivers)")
-        else:
+            feed_sidebar = False
+
+        if feed_sidebar:
             # Get default shortcut
             # There is no other way
             d = qt.QFileDialog(self)
@@ -453,9 +468,13 @@ class _FabioData(object):
     def shape(self):
         if self.__fabioFile.nframes == 0:
             return None
+        if self.__fabioFile.nframes == 1:
+            return [slice(None), slice(None)]
         return [self.__fabioFile.nframes, slice(None), slice(None)]
 
     def __getitem__(self, selector):
+        if self.__fabioFile.nframes == 1 and selector == tuple():
+            return self.__fabioFile.data
         if isinstance(selector, tuple) and len(selector) == 1:
             selector = selector[0]
 
@@ -527,6 +546,10 @@ class AbstractDataFileDialog(qt.QDialog):
     def _init(self):
         self.setWindowTitle("Open")
 
+        self.__openedFiles = []
+        """Store the list of files opened by the model itself."""
+        # FIXME: It should be managed one by one by Hdf5Item itself
+
         self.__directory = None
         self.__directoryLoadedFilter = None
         self.__errorWhileLoadingFile = None
@@ -576,10 +599,6 @@ class AbstractDataFileDialog(qt.QDialog):
         self.__fileTypeCombo.setCurrentIndex(0)
         self.__filterSelected(0)
 
-        self.__openedFiles = []
-        """Store the list of files opened by the model itself."""
-        # FIXME: It should be managed one by one by Hdf5Item itself
-
         # It is not possible to override the QObject destructor nor
         # to access to the content of the Python object with the `destroyed`
         # signal cause the Python method was already removed with the QWidget,
@@ -1023,15 +1042,16 @@ class AbstractDataFileDialog(qt.QDialog):
                 return
         self.__directoryLoadedFilter = path
         self.__processing += 1
+        if self.__fileModel is None:
+            return
         index = self.__fileModel.setRootPath(path)
         if not index.isValid():
+            # There is a problem with this path
+            # No asynchronous process will be waked up
             self.__processing -= 1
             self.__browser.setRootIndex(index, model=self.__fileModel)
             self.__clearData()
             self.__updatePath()
-        else:
-            # asynchronous process
-            pass
 
     def __directoryLoaded(self, path):
         if self.__directoryLoadedFilter is not None:
@@ -1040,6 +1060,8 @@ class AbstractDataFileDialog(qt.QDialog):
                 # The first click on the sidebar sent 2 events
                 self.__processing -= 1
                 return
+        if self.__fileModel is None:
+            return
         index = self.__fileModel.index(path)
         self.__browser.setRootIndex(index, model=self.__fileModel)
         self.__updatePath()
@@ -1061,8 +1083,6 @@ class AbstractDataFileDialog(qt.QDialog):
     def __openFabioFile(self, filename):
         self.__closeFile()
         try:
-            if fabio is None:
-                raise ImportError("Fabio module is not available")
             self.__fabio = fabio.open(filename)
             self.__openedFiles.append(self.__fabio)
             self.__selectedFile = filename
@@ -1108,10 +1128,10 @@ class AbstractDataFileDialog(qt.QDialog):
         if codec.is_autodetect():
             if self.__isSilxHavePriority(filename):
                 openners.append(self.__openSilxFile)
-                if fabio is not None and self._isFabioFilesSupported():
+                if self._isFabioFilesSupported():
                     openners.append(self.__openFabioFile)
             else:
-                if fabio is not None and self._isFabioFilesSupported():
+                if self._isFabioFilesSupported():
                     openners.append(self.__openFabioFile)
                 openners.append(self.__openSilxFile)
         elif codec.is_silx_codec():
@@ -1159,10 +1179,9 @@ class AbstractDataFileDialog(qt.QDialog):
                         is_fabio_have_priority = not codec.is_silx_codec() and not self.__isSilxHavePriority(path)
                         if is_fabio_decoder or is_fabio_have_priority:
                             # Then it's flat frame container
-                            if fabio is not None:
-                                self.__openFabioFile(path)
-                                if self.__fabio is not None:
-                                    selectedData = _FabioData(self.__fabio)
+                            self.__openFabioFile(path)
+                            if self.__fabio is not None:
+                                selectedData = _FabioData(self.__fabio)
             else:
                 assert(False)
 
@@ -1221,6 +1240,7 @@ class AbstractDataFileDialog(qt.QDialog):
         if self.__previewWidget is not None:
             self.__previewWidget.setData(None)
         if self.__selectorWidget is not None:
+            self.__selectorWidget.setData(None)
             self.__selectorWidget.hide()
         self.__selectedData = None
         self.__data = None
@@ -1238,6 +1258,8 @@ class AbstractDataFileDialog(qt.QDialog):
         If :meth:`_isDataSupported` returns false, this function will be
         inhibited and no data will be selected.
         """
+        if isinstance(data, _FabioData):
+            data = data[()]
         if self.__previewWidget is not None:
             fromDataSelector = self.__selectedData is not None
             self.__previewWidget.setData(data, fromDataSelector=fromDataSelector)
@@ -1305,8 +1327,10 @@ class AbstractDataFileDialog(qt.QDialog):
             filename = ""
             dataPath = None
 
-        if useSelectorWidget and self.__selectorWidget is not None and self.__selectorWidget.isVisible():
+        if useSelectorWidget and self.__selectorWidget is not None and self.__selectorWidget.isUsed():
             slicing = self.__selectorWidget.slicing()
+            if slicing == tuple():
+                slicing = None
         else:
             slicing = None
 
@@ -1471,9 +1495,7 @@ class AbstractDataFileDialog(qt.QDialog):
                     self.__clearData()
 
                 if self.__selectorWidget is not None:
-                    self.__selectorWidget.setVisible(url.data_slice() is not None)
-                    if url.data_slice() is not None:
-                        self.__selectorWidget.setSlicing(url.data_slice())
+                    self.__selectorWidget.selectSlicing(url.data_slice())
             else:
                 self.__errorWhileLoadingFile = (url.file_path(), "File not found")
                 self.__clearData()
