@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -210,7 +210,7 @@ class DataFileDialog(AbstractDataFileDialog):
     .. code-block:: python
 
         url = dialog.selectedDataUrl()
-        with h5py.File(url.file_path()) as h5:
+        with h5py.File(url.file_path(), mode="r") as h5:
             data = h5[url.data_path()]
     """
 
