@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -410,7 +410,7 @@ class CurvesROIWidget(qt.QWidget):
         """
         if visible:
             # if no ROI existing yet, add the default one
-            if self.roiTable.rowCount() is 0:
+            if self.roiTable.rowCount() == 0:
                 old = self.blockSignals(True)  # avoid several sigROISignal emission
                 self._add()
                 self.blockSignals(old)
@@ -703,7 +703,7 @@ class ROITable(TableWidget):
         remove the current active roi
         """
         activeItems = self.selectedItems()
-        if len(activeItems) is 0:
+        if len(activeItems) == 0:
             return
         old = self.blockSignals(True)  # avoid several emission of sigROISignal
         roiToRm = set()
@@ -1069,7 +1069,8 @@ class ROI(_RegionOfInterestBase):
     """Signal emitted when the ROI is edited"""
 
     def __init__(self, name, fromdata=None, todata=None, type_=None):
-        _RegionOfInterestBase.__init__(self, name=name)
+        _RegionOfInterestBase.__init__(self)
+        self.setName(name)
         global _indexNextROI
         self._id = _indexNextROI
         _indexNextROI += 1
@@ -1214,14 +1215,14 @@ class ROI(_RegionOfInterestBase):
         if len(idx):
             xw = x[idx]
             yw = y[idx]
-            rawCounts = yw.sum(dtype=numpy.float)
+            rawCounts = yw.sum(dtype=numpy.float64)
             deltaX = xw[-1] - xw[0]
             deltaY = yw[-1] - yw[0]
             if deltaX > 0.0:
                 slope = (deltaY / deltaX)
                 background = yw[0] + slope * (xw - xw[0])
                 netCounts = (rawCounts -
-                             background.sum(dtype=numpy.float))
+                             background.sum(dtype=numpy.float64))
             else:
                 netCounts = 0.0
         else:
@@ -1255,7 +1256,7 @@ class ROI(_RegionOfInterestBase):
         y = y[(x >= self._fromdata) & (x <= self._todata)]
         x = x[(x >= self._fromdata) & (x <= self._todata)]
 
-        if x.size is 0:
+        if x.size == 0:
             return 0.0, 0.0
 
         rawArea = numpy.trapz(y, x=x)
@@ -1268,6 +1269,10 @@ class ROI(_RegionOfInterestBase):
         netArea = rawArea - background
         return rawArea, netArea
 
+    @docstring(_RegionOfInterestBase)
+    def contains(self, position):
+        return self._fromdata <= position[0] <= self._todata
+
 
 class _RoiMarkerManager(object):
     """
