@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -37,7 +37,7 @@ from __future__ import division
 
 __authors__ = ["V.A. Sole", "T. Vincent", "P. Knobel"]
 __license__ = "MIT"
-__date__ = "12/07/2018"
+__date__ = "25/09/2020"
 
 from . import PlotAction
 from silx.io.utils import save1D, savespec, NEXUS_HDF5_EXT
@@ -131,6 +131,7 @@ class SaveAction(PlotAction):
     IMAGE_FILTER_CSV_TAB = 'Image data as tab-separated CSV (*.csv)'
     IMAGE_FILTER_RGB_PNG = 'Image as PNG (*.png)'
     IMAGE_FILTER_NXDATA = 'Image as NXdata (%s)' % _NEXUS_HDF5_EXT_STR
+
     DEFAULT_IMAGE_FILTERS = (IMAGE_FILTER_EDF,
                              IMAGE_FILTER_TIFF,
                              IMAGE_FILTER_NUMPY,
@@ -156,6 +157,8 @@ class SaveAction(PlotAction):
             'image': OrderedDict(),
             'scatter': OrderedDict()}
 
+        self._appendFilters = list(self.DEFAULT_APPEND_FILTERS)
+
         # Initialize filters
         for nameFilter in self.DEFAULT_ALL_FILTERS:
             self.setFileFilter(
@@ -185,10 +188,11 @@ class SaveAction(PlotAction):
         self.setShortcut(qt.QKeySequence.Save)
         self.setShortcutContext(qt.Qt.WidgetShortcut)
 
-    def _errorMessage(self, informativeText=''):
+    @staticmethod
+    def _errorMessage(informativeText='', parent=None):
         """Display an error message."""
         # TODO issue with QMessageBox size fixed and too small
-        msg = qt.QMessageBox(self.plot)
+        msg = qt.QMessageBox(parent)
         msg.setIcon(qt.QMessageBox.Critical)
         msg.setInformativeText(informativeText + ' ' + str(sys.exc_info()[1]))
         msg.setDetailedText(traceback.format_exc())
@@ -220,7 +224,45 @@ class SaveAction(PlotAction):
         ylabel = item.getYLabel() or self.plot.getYAxis().getLabel()
         return xlabel, ylabel
 
-    def _selectWriteableOutputGroup(self, filename):
+    def _get1dData(self, item):
+        "provide xdata, [ydata], xlabel, [ylabel] and manages error bars"
+        xlabel, ylabel = self._getAxesLabels(item)
+        x_data = item.getXData(copy=False)
+        y_data = item.getYData(copy=False)
+        x_err = item.getXErrorData(copy=False)
+        y_err = item.getYErrorData(copy=False)
+        labels = [ylabel]
+        data = [y_data]
+
+        if x_err is not None:
+            if numpy.isscalar(x_err):
+                data.append(numpy.zeros_like(y_data) + x_err)
+                labels.append(xlabel + "_errors")
+            elif x_err.ndim == 1:
+                data.append(x_err)
+                labels.append(xlabel + "_errors")
+            elif x_err.ndim == 2:
+                data.append(x_err[0])
+                labels.append(xlabel + "_errors_below")
+                data.append(x_err[1])
+                labels.append(xlabel + "_errors_above")
+
+        if y_err is not None:
+            if numpy.isscalar(y_err):
+                data.append(numpy.zeros_like(y_data) + y_err)
+                labels.append(ylabel + "_errors")
+            elif y_err.ndim == 1:
+                data.append(y_err)
+                labels.append(ylabel + "_errors")
+            elif y_err.ndim == 2:
+                data.append(y_err[0])
+                labels.append(ylabel + "_errors_below")
+                data.append(y_err[1])
+                labels.append(ylabel + "_errors_above")
+        return x_data, data, xlabel, labels
+
+    @staticmethod
+    def _selectWriteableOutputGroup(filename, parent):
         if os.path.exists(filename) and os.path.isfile(filename) \
                 and os.access(filename, os.W_OK):
             entryPath = selectOutputGroup(filename)
@@ -232,11 +274,11 @@ class SaveAction(PlotAction):
             # create new entry in new file
             return "/entry"
         else:
-            self._errorMessage('Save failed (file access issue)\n')
+            SaveAction._errorMessage('Save failed (file access issue)\n', parent=parent)
             return None
 
     def _saveCurveAsNXdata(self, curve, filename):
-        entryPath = self._selectWriteableOutputGroup(filename)
+        entryPath = self._selectWriteableOutputGroup(filename, parent=self.plot)
         if entryPath is None:
             return False
 
@@ -273,7 +315,7 @@ class SaveAction(PlotAction):
         if curve is None:
             curves = plot.getAllCurves()
             if not curves:
-                self._errorMessage("No curve to be saved")
+                self._errorMessage("No curve to be saved", parent=self.plot)
                 return False
             curve = curves[0]
 
@@ -286,20 +328,19 @@ class SaveAction(PlotAction):
             # .npy or nxdata
             fmt, csvdelim, autoheader = ("", "", False)
 
-        xlabel, ylabel = self._getAxesLabels(curve)
-
         if nameFilter == self.CURVE_FILTER_NXDATA:
             return self._saveCurveAsNXdata(curve, filename)
 
+        xdata, data, xlabel, labels = self._get1dData(curve)
+
         try:
             save1D(filename,
-                   curve.getXData(copy=False),
-                   curve.getYData(copy=False),
-                   xlabel, [ylabel],
+                   xdata, data,
+                   xlabel, labels,
                    fmt=fmt, csvdelim=csvdelim,
                    autoheader=autoheader)
         except IOError:
-            self._errorMessage('Save failed\n')
+            self._errorMessage('Save failed\n', parent=self.plot)
             return False
 
         return True
@@ -317,41 +358,36 @@ class SaveAction(PlotAction):
 
         curves = plot.getAllCurves()
         if not curves:
-            self._errorMessage("No curves to be saved")
+            self._errorMessage("No curves to be saved", parent=self.plot)
             return False
 
         curve = curves[0]
         scanno = 1
         try:
-            xlabel = curve.getXLabel() or plot.getGraphXLabel()
-            ylabel = curve.getYLabel() or plot.getGraphYLabel(curve.getYAxis())
+            xdata, data, xlabel, labels = self._get1dData(curve)
+
             specfile = savespec(filename,
-                                curve.getXData(copy=False),
-                                curve.getYData(copy=False),
-                                xlabel,
-                                ylabel,
+                                xdata, data,
+                                xlabel, labels,
                                 fmt="%.7g", scan_number=1, mode="w",
                                 write_file_header=True,
                                 close_file=False)
         except IOError:
-            self._errorMessage('Save failed\n')
+            self._errorMessage('Save failed\n', parent=self.plot)
             return False
 
         for curve in curves[1:]:
             try:
                 scanno += 1
-                xlabel = curve.getXLabel() or plot.getGraphXLabel()
-                ylabel = curve.getYLabel() or plot.getGraphYLabel(curve.getYAxis())
+                xdata, data, xlabel, labels = self._get1dData(curve)
                 specfile = savespec(specfile,
-                                    curve.getXData(copy=False),
-                                    curve.getYData(copy=False),
-                                    xlabel,
-                                    ylabel,
+                                    xdata, data,
+                                    xlabel, labels,
                                     fmt="%.7g", scan_number=scanno,
                                     write_file_header=False,
                                     close_file=False)
             except IOError:
-                self._errorMessage('Save failed\n')
+                self._errorMessage('Save failed\n', parent=self.plot)
                 return False
         specfile.close()
 
@@ -391,12 +427,12 @@ class SaveAction(PlotAction):
             try:
                 numpy.save(filename, data)
             except IOError:
-                self._errorMessage('Save failed\n')
+                self._errorMessage('Save failed\n', parent=self.plot)
                 return False
             return True
 
         elif nameFilter == self.IMAGE_FILTER_NXDATA:
-            entryPath = self._selectWriteableOutputGroup(filename)
+            entryPath = self._selectWriteableOutputGroup(filename, parent=self.plot)
             if entryPath is None:
                 return False
             xorigin, yorigin = image.getOrigin()
@@ -438,7 +474,7 @@ class SaveAction(PlotAction):
                        autoheader=True)
 
             except IOError:
-                self._errorMessage('Save failed\n')
+                self._errorMessage('Save failed\n', parent=self.plot)
                 return False
             return True
 
@@ -471,7 +507,7 @@ class SaveAction(PlotAction):
             return False
 
         if nameFilter == self.SCATTER_FILTER_NXDATA:
-            entryPath = self._selectWriteableOutputGroup(filename)
+            entryPath = self._selectWriteableOutputGroup(filename, parent=self.plot)
             if entryPath is None:
                 return False
             scatter = plot.getScatter()
@@ -502,7 +538,7 @@ class SaveAction(PlotAction):
                 axes_errors=[xerror, yerror],
                 title=plot.getGraphTitle())
 
-    def setFileFilter(self, dataKind, nameFilter, func, index=None):
+    def setFileFilter(self, dataKind, nameFilter, func, index=None, appendToFile=False):
         """Set a name filter to add/replace a file format support
 
         :param str dataKind:
@@ -513,10 +549,15 @@ class SaveAction(PlotAction):
         :param callable func: The function to call to perform saving.
             Expected signature is:
             bool func(PlotWidget plot, str filename, str nameFilter)
+        :param bool appendToFile: True to append the data into the selected
+            file.
         :param integer index: Index of the filter in the final list (or None)
         """
         assert dataKind in ('all', 'curve', 'curves', 'image', 'scatter')
 
+        if appendToFile:
+            self._appendFilters.append(nameFilter)
+
         # first append or replace the new filter to prevent colissions
         self._filters[dataKind][nameFilter] = func
         if index is None:
@@ -601,7 +642,7 @@ class SaveAction(PlotAction):
         def onFilterSelection(filt_):
             # disable overwrite confirmation for NXdata types,
             # because we append the data to existing files
-            if filt_ in self.DEFAULT_APPEND_FILTERS:
+            if filt_ in self._appendFilters:
                 dialog.setOption(dialog.DontConfirmOverwrite)
             else:
                 dialog.setOption(dialog.DontConfirmOverwrite, False)
@@ -619,7 +660,7 @@ class SaveAction(PlotAction):
             # Check for correct file extension
             # Extract file extensions as .something
             extensions = [ext[ext.find('.'):] for ext in
-                          nameFilter[nameFilter.find('(')+1:-1].split()]
+                          nameFilter[nameFilter.find('(') + 1:-1].split()]
             for ext in extensions:
                 if (len(filename) > len(ext) and
                         filename[-len(ext):].lower() == ext.lower()):
