@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -33,6 +33,7 @@ __date__ = "21/12/2018"
 
 import logging
 import datetime as dt
+from typing import Tuple
 import numpy
 
 from pkg_resources import parse_version as _parse_version
@@ -44,7 +45,7 @@ _logger = logging.getLogger(__name__)
 from ... import qt
 
 # First of all init matplotlib and set its backend
-from ..matplotlib import FigureCanvasQTAgg
+from ...utils.matplotlib import FigureCanvasQTAgg
 import matplotlib
 from matplotlib.container import Container
 from matplotlib.figure import Figure
@@ -52,6 +53,7 @@ from matplotlib.patches import Rectangle, Polygon
 from matplotlib.image import AxesImage
 from matplotlib.backend_bases import MouseEvent
 from matplotlib.lines import Line2D
+from matplotlib.text import Text
 from matplotlib.collections import PathCollection, LineCollection
 from matplotlib.ticker import Formatter, ScalarFormatter, Locator
 from matplotlib.tri import Triangulation
@@ -252,6 +254,60 @@ class _PickableContainer(Container):
         return False, {}
 
 
+class _TextWithOffset(Text):
+    """Text object which can be displayed at a specific position
+    of the plot, but with a pixel offset"""
+
+    def __init__(self, *args, **kwargs):
+        Text.__init__(self, *args, **kwargs)
+        self.pixel_offset = (0, 0)
+        self.__cache = None
+
+    def draw(self, renderer):
+        self.__cache = None
+        return Text.draw(self, renderer)
+
+    def __get_xy(self):
+        if self.__cache is not None:
+            return self.__cache
+
+        align = self.get_horizontalalignment()
+        if align == "left":
+            xoffset = self.pixel_offset[0]
+        elif align == "right":
+            xoffset = -self.pixel_offset[0]
+        else:
+            xoffset = 0
+
+        align = self.get_verticalalignment()
+        if align == "top":
+            yoffset = -self.pixel_offset[1]
+        elif align == "bottom":
+            yoffset = self.pixel_offset[1]
+        else:
+            yoffset = 0
+
+        trans = self.get_transform()
+        invtrans = self.get_transform().inverted()
+
+        x = super(_TextWithOffset, self).convert_xunits(self._x)
+        y = super(_TextWithOffset, self).convert_xunits(self._y)
+        pos = x, y
+        proj = trans.transform_point(pos)
+        proj = proj + numpy.array((xoffset, yoffset))
+        pos = invtrans.transform_point(proj)
+        self.__cache = pos
+        return pos
+
+    def convert_xunits(self, x):
+        """Return the pixel position of the annotated point."""
+        return self.__get_xy()[0]
+
+    def convert_yunits(self, y):
+        """Return the pixel position of the annotated point."""
+        return self.__get_xy()[1]
+
+
 class _MarkerContainer(_PickableContainer):
     """Marker artists container supporting draw/remove and text position update
 
@@ -263,9 +319,10 @@ class _MarkerContainer(_PickableContainer):
     :param y: Y coordinate of the marker (None for vertical lines)
     """
 
-    def __init__(self, artists, x, y, yAxis):
+    def __init__(self, artists, symbol, x, y, yAxis):
         self.line = artists[0]
         self.text = artists[1] if len(artists) > 1 else None
+        self.symbol = symbol
         self.x = x
         self.y = y
         self.yAxis = yAxis
@@ -278,27 +335,39 @@ class _MarkerContainer(_PickableContainer):
         if self.text is not None:
             self.text.draw(*args, **kwargs)
 
-    def updateMarkerText(self, xmin, xmax, ymin, ymax):
+    def updateMarkerText(self, xmin, xmax, ymin, ymax, yinverted):
         """Update marker text position and visibility according to plot limits
 
         :param xmin: X axis lower limit
         :param xmax: X axis upper limit
         :param ymin: Y axis lower limit
-        :param ymax: Y axis upprt limit
+        :param ymax: Y axis upper limit
+        :param yinverted: True if the y axis is inverted
         """
         if self.text is not None:
             visible = ((self.x is None or xmin <= self.x <= xmax) and
                        (self.y is None or ymin <= self.y <= ymax))
             self.text.set_visible(visible)
 
-            if self.x is not None and self.y is None:  # vertical line
-                delta = abs(ymax - ymin)
-                if ymin > ymax:
-                    ymax = ymin
-                ymax -= 0.005 * delta
-                self.text.set_y(ymax)
+            if self.x is not None and self.y is not None:
+                if self.symbol is None:
+                    valign = 'baseline'
+                else:
+                    if yinverted:
+                        valign = 'bottom'
+                    else:
+                        valign = 'top'
+                self.text.set_verticalalignment(valign)
 
-            if self.x is None and self.y is not None:  # Horizontal line
+            elif self.y is None:  # vertical line
+                # Always display it on top
+                center = (ymax + ymin) * 0.5
+                pos = (ymax - ymin) * 0.5 * 0.99
+                if yinverted:
+                    pos = -pos
+                self.text.set_y(center + pos)
+
+            elif self.x is None:  # Horizontal line
                 delta = abs(xmax - xmin)
                 if xmin > xmax:
                     xmax = xmin
@@ -354,9 +423,35 @@ class _DoubleColoredLinePatch(matplotlib.patches.Patch):
 
 
 class Image(AxesImage):
-    """An AxesImage with a fast path for uint8 RGBA images"""
+    """An AxesImage with a fast path for uint8 RGBA images.
+
+    :param List[float] silx_origin: (ox, oy) Offset of the image.
+    :param List[float] silx_scale: (sx, sy) Scale of the image.
+    """
+
+    def __init__(self, *args,
+                 silx_origin=(0., 0.),
+                 silx_scale=(1., 1.),
+                 **kwargs):
+        super().__init__(*args, **kwargs)
+        self.__silx_origin = silx_origin
+        self.__silx_scale = silx_scale
+
+    def contains(self, mouseevent):
+        """Overridden to fill 'ind' with row and column"""
+        inside, info = super().contains(mouseevent)
+        if inside:
+            x, y = mouseevent.xdata, mouseevent.ydata
+            ox, oy = self.__silx_origin
+            sx, sy = self.__silx_scale
+            height, width = self.get_size()
+            column = numpy.clip(int((x - ox) / sx), 0, width - 1)
+            row = numpy.clip(int((y - oy) / sy), 0, height - 1)
+            info['ind'] = (row,), (column,)
+        return inside, info
 
     def set_data(self, A):
+        """Overridden to add a fast path for RGBA unit8 images"""
         A = numpy.array(A, copy=False)
         if A.ndim != 3 or A.shape[2] != 4 or A.dtype != numpy.uint8:
             super(Image, self).set_data(A)
@@ -402,10 +497,15 @@ class BackendMatplotlib(BackendBase.BackendBase):
 
         # disable the use of offsets
         try:
-            self.ax.get_yaxis().get_major_formatter().set_useOffset(False)
-            self.ax.get_xaxis().get_major_formatter().set_useOffset(False)
-            self.ax2.get_yaxis().get_major_formatter().set_useOffset(False)
-            self.ax2.get_xaxis().get_major_formatter().set_useOffset(False)
+            axes = [
+                self.ax.get_yaxis().get_major_formatter(),
+                self.ax.get_xaxis().get_major_formatter(),
+                self.ax2.get_yaxis().get_major_formatter(),
+                self.ax2.get_xaxis().get_major_formatter(),
+            ]
+            for axis in axes:
+                axis.set_useOffset(False)
+                axis.set_scientific(False)
         except:
             _logger.warning('Cannot disabled axes offsets in %s '
                             % matplotlib.__version__)
@@ -485,16 +585,16 @@ class BackendMatplotlib(BackendBase.BackendBase):
     def addCurve(self, x, y,
                  color, symbol, linewidth, linestyle,
                  yaxis,
-                 xerror, yerror, z,
+                 xerror, yerror,
                  fill, alpha, symbolsize, baseline):
         for parameter in (x, y, color, symbol, linewidth, linestyle,
-                          yaxis, z, fill, alpha, symbolsize):
+                          yaxis, fill, alpha, symbolsize):
             assert parameter is not None
         assert yaxis in ('left', 'right')
 
         if (len(color) == 4 and
                 type(color[3]) in [type(1), numpy.uint8, numpy.int8]):
-            color = numpy.array(color, dtype=numpy.float) / 255.
+            color = numpy.array(color, dtype=numpy.float64) / 255.
 
         if yaxis == "right":
             axes = self.ax2
@@ -502,7 +602,7 @@ class BackendMatplotlib(BackendBase.BackendBase):
         else:
             axes = self.ax
 
-        picker = 3
+        pickradius = 3
 
         artists = []  # All the artists composing the curve
 
@@ -528,7 +628,7 @@ class BackendMatplotlib(BackendBase.BackendBase):
 
         if hasattr(color, 'dtype') and len(color) == len(x):
             # scatter plot
-            if color.dtype not in [numpy.float32, numpy.float]:
+            if color.dtype not in [numpy.float32, numpy.float64]:
                 actualColor = color / 255.
             else:
                 actualColor = color
@@ -540,7 +640,8 @@ class BackendMatplotlib(BackendBase.BackendBase):
                                       linestyle=linestyle,
                                       color=actualColor[0],
                                       linewidth=linewidth,
-                                      picker=picker,
+                                      picker=True,
+                                      pickradius=pickradius,
                                       marker=None)
                 artists += list(curveList)
 
@@ -548,7 +649,8 @@ class BackendMatplotlib(BackendBase.BackendBase):
             scatter = axes.scatter(x, y,
                                    color=actualColor,
                                    marker=marker,
-                                   picker=picker,
+                                   picker=True,
+                                   pickradius=pickradius,
                                    s=symbolsize**2)
             artists.append(scatter)
 
@@ -566,7 +668,8 @@ class BackendMatplotlib(BackendBase.BackendBase):
                                   color=color,
                                   linewidth=linewidth,
                                   marker=symbol,
-                                  picker=picker,
+                                  picker=True,
+                                  pickradius=pickradius,
                                   markersize=symbolsize)
             artists += list(curveList)
 
@@ -584,12 +687,12 @@ class BackendMatplotlib(BackendBase.BackendBase):
 
         return _PickableContainer(artists)
 
-    def addImage(self, data, origin, scale, z, colormap, alpha):
+    def addImage(self, data, origin, scale, colormap, alpha):
         # Non-uniform image
         # http://wiki.scipy.org/Cookbook/Histograms
         # Non-linear axes
         # http://stackoverflow.com/questions/11488800/non-linear-axes-for-imshow-in-matplotlib
-        for parameter in (data, origin, scale, z):
+        for parameter in (data, origin, scale):
             assert parameter is not None
 
         origin = float(origin[0]), float(origin[1])
@@ -600,7 +703,9 @@ class BackendMatplotlib(BackendBase.BackendBase):
         image = Image(self.ax,
                       interpolation='nearest',
                       picker=True,
-                      origin='lower')
+                      origin='lower',
+                      silx_origin=origin,
+                      silx_scale=scale)
 
         if alpha < 1:
             image.set_alpha(alpha)
@@ -627,32 +732,36 @@ class BackendMatplotlib(BackendBase.BackendBase):
 
         if data.ndim == 2:  # Data image, convert to RGBA image
             data = colormap.applyToData(data)
-
+        elif data.dtype == numpy.uint16:
+            # Normalize uint16 data to have a similar behavior as opengl backend
+            data = data.astype(numpy.float32)
+            data /= 65535
+        
         image.set_data(data)
         self.ax.add_artist(image)
         return image
 
-    def addTriangles(self, x, y, triangles, color, z, alpha):
-        for parameter in (x, y, triangles, color, z, alpha):
+    def addTriangles(self, x, y, triangles, color, alpha):
+        for parameter in (x, y, triangles, color, alpha):
             assert parameter is not None
 
         color = numpy.array(color, copy=False)
         assert color.ndim == 2 and len(color) == len(x)
 
-        if color.dtype not in [numpy.float32, numpy.float]:
+        if color.dtype not in [numpy.float32, numpy.float64]:
             color = color.astype(numpy.float32) / 255.
 
         collection = TriMesh(
             Triangulation(x, y, triangles),
             alpha=alpha,
-            picker=0)  # 0 enables picking on filled triangle
+            pickradius=0)  # 0 enables picking on filled triangle
         collection.set_color(color)
         self.ax.add_collection(collection)
 
         return collection
 
-    def addItem(self, x, y, shape, color, fill, overlay, z,
-                linestyle, linewidth, linebgcolor):
+    def addShape(self, x, y, shape, color, fill, overlay,
+                 linestyle, linewidth, linebgcolor):
         if (linebgcolor is not None and
                 shape not in ('rectangle', 'polygon', 'polylines')):
             _logger.warning(
@@ -755,17 +864,11 @@ class BackendMatplotlib(BackendBase.BackendBase):
                            markersize=10.)[-1]
 
             if text is not None:
-                if symbol is None:
-                    valign = 'baseline'
-                else:
-                    valign = 'top'
-                    text = "  " + text
-
-                textArtist = ax.text(x, y, text,
-                                     color=color,
-                                     horizontalalignment='left',
-                                     verticalalignment=valign)
-
+                textArtist = _TextWithOffset(x, y, text,
+                                             color=color,
+                                             horizontalalignment='left')
+                if symbol is not None:
+                    textArtist.pixel_offset = 10, 3
         elif x is not None:
             line = ax.axvline(x,
                               color=color,
@@ -773,11 +876,11 @@ class BackendMatplotlib(BackendBase.BackendBase):
                               linestyle=linestyle)
             if text is not None:
                 # Y position will be updated in updateMarkerText call
-                textArtist = ax.text(x, 1., " " + text,
-                                     color=color,
-                                     horizontalalignment='left',
-                                     verticalalignment='top')
-
+                textArtist = _TextWithOffset(x, 1., text,
+                                             color=color,
+                                             horizontalalignment='left',
+                                             verticalalignment='top')
+                textArtist.pixel_offset = 5, 3
         elif y is not None:
             line = ax.axhline(y,
                               color=color,
@@ -786,24 +889,26 @@ class BackendMatplotlib(BackendBase.BackendBase):
 
             if text is not None:
                 # X position will be updated in updateMarkerText call
-                textArtist = ax.text(1., y, " " + text,
-                                     color=color,
-                                     horizontalalignment='right',
-                                     verticalalignment='top')
-
+                textArtist = _TextWithOffset(1., y, text,
+                                             color=color,
+                                             horizontalalignment='right',
+                                             verticalalignment='top')
+                textArtist.pixel_offset = 5, 3
         else:
             raise RuntimeError('A marker must at least have one coordinate')
 
-        line.set_picker(5)
+        line.set_picker(True)
+        line.set_pickradius(5)
 
         # All markers are overlays
         line.set_animated(True)
         if textArtist is not None:
+            ax.add_artist(textArtist)
             textArtist.set_animated(True)
 
         artists = [line] if textArtist is None else [line, textArtist]
-        container = _MarkerContainer(artists, x, y, yaxis)
-        container.updateMarkerText(xmin, xmax, ymin, ymax)
+        container = _MarkerContainer(artists, symbol, x, y, yaxis)
+        container.updateMarkerText(xmin, xmax, ymin, ymax, self.isYAxisInverted())
 
         return container
 
@@ -811,12 +916,13 @@ class BackendMatplotlib(BackendBase.BackendBase):
         xmin, xmax = self.ax.get_xbound()
         ymin1, ymax1 = self.ax.get_ybound()
         ymin2, ymax2 = self.ax2.get_ybound()
+        yinverted = self.isYAxisInverted()
         for item in self._overlayItems():
             if isinstance(item, _MarkerContainer):
                 if item.yAxis == 'left':
-                    item.updateMarkerText(xmin, xmax, ymin1, ymax1)
+                    item.updateMarkerText(xmin, xmax, ymin1, ymax1, yinverted)
                 else:
-                    item.updateMarkerText(xmin, xmax, ymin2, ymax2)
+                    item.updateMarkerText(xmin, xmax, ymin2, ymax2, yinverted)
 
     # Remove methods
 
@@ -913,7 +1019,11 @@ class BackendMatplotlib(BackendBase.BackendBase):
             lambda item: item.isVisible() and item._backendRenderer is not None)
         count = len(items)
         for index, item in enumerate(items):
-            zorder = 1. + index / count
+            if item.getZValue() < 0.5:
+                # Make sure matplotlib z order is below the grid (with z=0.5)
+                zorder = 0.5 * index / count
+            else:  # Make sure matplotlib z order is above the grid (> 0.5)
+                zorder = 1. + index / count
             if zorder != item._backendRenderer.get_zorder():
                 item._backendRenderer.set_zorder(zorder)
 
@@ -1076,6 +1186,7 @@ class BackendMatplotlib(BackendBase.BackendBase):
     def setYAxisInverted(self, flag):
         if self.ax.yaxis_inverted() != bool(flag):
             self.ax.invert_yaxis()
+            self._updateMarkers()
 
     def isYAxisInverted(self):
         return self.ax.yaxis_inverted()
@@ -1094,67 +1205,58 @@ class BackendMatplotlib(BackendBase.BackendBase):
 
     # Data <-> Pixel coordinates conversion
 
-    def _mplQtYAxisCoordConversion(self, y, asint=True):
-        """Qt origin (top) to/from matplotlib origin (bottom) conversion.
+    def _getDevicePixelRatio(self) -> float:
+        """Compatibility wrapper for devicePixelRatioF"""
+        return 1.
 
-        :param y:
-        :param bool asint: True to cast to int, False to keep as float
-
-        :rtype: float
+    def _mplToQtPosition(self, x: float, y: float) -> Tuple[float, float]:
+        """Convert matplotlib "display" space coord to Qt widget logical pixel
         """
-        value = self.fig.get_window_extent().height - y
-        return int(value) if asint else value
+        ratio = self._getDevicePixelRatio()
+        # Convert from matplotlib origin (bottom) to Qt origin (top)
+        # and apply device pixel ratio
+        return x / ratio, (self.fig.get_window_extent().height - y) / ratio
+
+    def _qtToMplPosition(self, x: float, y: float) -> Tuple[float, float]:
+        """Convert Qt widget logical pixel to matplotlib "display" space coord
+        """
+        ratio = self._getDevicePixelRatio()
+        # Apply device pixel ration and
+        # convert from Qt origin (top) to matplotlib origin (bottom)
+        return x * ratio, self.fig.get_window_extent().height - (y * ratio)
 
     def dataToPixel(self, x, y, axis):
         ax = self.ax2 if axis == "right" else self.ax
-
-        pixels = ax.transData.transform_point((x, y))
-        xPixel, yPixel = pixels.T
-
-        # Convert from matplotlib origin (bottom) to Qt origin (top)
-        yPixel = self._mplQtYAxisCoordConversion(yPixel, asint=False)
-
-        return xPixel, yPixel
+        displayPos = ax.transData.transform_point((x, y)).transpose()
+        return self._mplToQtPosition(*displayPos)
 
     def pixelToData(self, x, y, axis):
         ax = self.ax2 if axis == "right" else self.ax
-
-        # Convert from Qt origin (top) to matplotlib origin (bottom)
-        y = self._mplQtYAxisCoordConversion(y, asint=False)
-
-        inv = ax.transData.inverted()
-        x, y = inv.transform_point((x, y))
-        return x, y
+        displayPos = self._qtToMplPosition(x, y)
+        return tuple(ax.transData.inverted().transform_point(displayPos))
 
     def getPlotBoundsInPixels(self):
         bbox = self.ax.get_window_extent()
         # Warning this is not returning int...
-        return (int(bbox.xmin),
-                self._mplQtYAxisCoordConversion(bbox.ymax, asint=True),
-                int(bbox.width),
-                int(bbox.height))
+        ratio = self._getDevicePixelRatio()
+        return tuple(int(value / ratio) for value in (
+            bbox.xmin,
+            self.fig.get_window_extent().height - bbox.ymax,
+            bbox.width,
+            bbox.height))
 
-    def setAxesDisplayed(self, displayed):
-        """Display or not the axes.
+    def setAxesMargins(self, left: float, top: float, right: float, bottom: float):
+        width, height = 1. - left - right, 1. - top - bottom
+        position = left, bottom, width, height
+
+        # Toggle display of axes and viewbox rect
+        isFrameOn = position != (0., 0., 1., 1.)
+        self.ax.set_frame_on(isFrameOn)
+        self.ax2.set_frame_on(isFrameOn)
+
+        self.ax.set_position(position)
+        self.ax2.set_position(position)
 
-        :param bool displayed: If `True` axes are displayed. If `False` axes
-            are not anymore visible and the margin used for them is removed.
-        """
-        BackendBase.BackendBase.setAxesDisplayed(self, displayed)
-        if displayed:
-            # show axes and viewbox rect
-            self.ax.set_axis_on()
-            self.ax2.set_axis_on()
-            # set the default margins
-            self.ax.set_position([.15, .15, .75, .75])
-            self.ax2.set_position([.15, .15, .75, .75])
-        else:
-            # hide axes and viewbox rect
-            self.ax.set_axis_off()
-            self.ax2.set_axis_off()
-            # remove external margins
-            self.ax.set_position([0, 0, 1, 1])
-            self.ax2.set_position([0, 0, 1, 1])
         self._synchronizeBackgroundColors()
         self._synchronizeForegroundColors()
         self._plot._setDirtyPlot()
@@ -1168,7 +1270,7 @@ class BackendMatplotlib(BackendBase.BackendBase):
         else:
             dataBackgroundColor = backgroundColor
 
-        if self.ax.axison:
+        if self.ax.get_frame_on():
             self.fig.patch.set_facecolor(backgroundColor)
             if self._matplotlibVersion < _parse_version('2'):
                 self.ax.set_axis_bgcolor(dataBackgroundColor)
@@ -1187,7 +1289,7 @@ class BackendMatplotlib(BackendBase.BackendBase):
             gridColor = foregroundColor
 
         for axes in (self.ax, self.ax2):
-            if axes.axison:
+            if axes.get_frame_on():
                 axes.spines['bottom'].set_color(foregroundColor)
                 axes.spines['top'].set_color(foregroundColor)
                 axes.spines['right'].set_color(foregroundColor)
@@ -1244,14 +1346,18 @@ class BackendMatplotlibQt(FigureCanvasQTAgg, BackendMatplotlib):
         self.mpl_connect('motion_notify_event', self._onMouseMove)
         self.mpl_connect('scroll_event', self._onMouseWheel)
 
-    def contextMenuEvent(self, event):
-        """Override QWidget.contextMenuEvent to implement the context menu"""
-        # Makes sure it is overridden (issue with PySide)
-        BackendBase.BackendBase.contextMenuEvent(self, event)
-
     def postRedisplay(self):
         self._sigPostRedisplay.emit()
 
+    def _getDevicePixelRatio(self) -> float:
+        """Compatibility wrapper for devicePixelRatioF"""
+        if hasattr(self, 'devicePixelRatioF'):
+            ratio = self.devicePixelRatioF()
+        else:  # Qt < 5.6 compatibility
+            ratio = float(self.devicePixelRatio())
+        # Safety net: avoid returning 0
+        return ratio if ratio != 0. else 1.
+
     # Mouse event forwarding
 
     _MPL_TO_PLOT_BUTTONS = {1: 'left', 2: 'middle', 3: 'right'}
@@ -1259,49 +1365,58 @@ class BackendMatplotlibQt(FigureCanvasQTAgg, BackendMatplotlib):
     def _onMousePress(self, event):
         button = self._MPL_TO_PLOT_BUTTONS.get(event.button, None)
         if button is not None:
-            self._plot.onMousePress(
-                event.x, self._mplQtYAxisCoordConversion(event.y),
-                button)
+            x, y = self._mplToQtPosition(event.x, event.y)
+            self._plot.onMousePress(int(x), int(y), button)
 
     def _onMouseMove(self, event):
+        x, y = self._mplToQtPosition(event.x, event.y)
         if self._graphCursor:
+            position = self._plot.pixelToData(
+                x, y, axis='left', check=True)
             lineh, linev = self._graphCursor
-            if event.inaxes not in (self.ax, self.ax2) and lineh.get_visible():
-                lineh.set_visible(False)
-                linev.set_visible(False)
-                self._plot._setDirtyPlot(overlayOnly=True)
-            else:
+            if position is not None:
                 linev.set_visible(True)
-                linev.set_xdata((event.xdata, event.xdata))
+                linev.set_xdata((position[0], position[0]))
                 lineh.set_visible(True)
-                lineh.set_ydata((event.ydata, event.ydata))
+                lineh.set_ydata((position[1], position[1]))
                 self._plot._setDirtyPlot(overlayOnly=True)
+            elif lineh.get_visible():
+                    lineh.set_visible(False)
+                    linev.set_visible(False)
+                    self._plot._setDirtyPlot(overlayOnly=True)
             # onMouseMove must trigger replot if dirty flag is raised
 
-        self._plot.onMouseMove(
-            event.x, self._mplQtYAxisCoordConversion(event.y))
+        self._plot.onMouseMove(int(x), int(y))
 
     def _onMouseRelease(self, event):
         button = self._MPL_TO_PLOT_BUTTONS.get(event.button, None)
         if button is not None:
-            self._plot.onMouseRelease(
-                event.x, self._mplQtYAxisCoordConversion(event.y),
-                button)
+            x, y = self._mplToQtPosition(event.x, event.y)
+            self._plot.onMouseRelease(int(x), int(y), button)
 
     def _onMouseWheel(self, event):
-        self._plot.onMouseWheel(
-            event.x, self._mplQtYAxisCoordConversion(event.y), event.step)
+        x, y = self._mplToQtPosition(event.x, event.y)
+        self._plot.onMouseWheel(int(x), int(y), event.step)
 
     def leaveEvent(self, event):
         """QWidget event handler"""
-        self._plot.onMouseLeaveWidget()
+        try:
+            plot = self._plot
+        except RuntimeError:
+            pass
+        else:
+            plot.onMouseLeaveWidget()
 
     # picking
 
     def pickItem(self, x, y, item):
+        xDisplay, yDisplay = self._qtToMplPosition(x, y)
         mouseEvent = MouseEvent(
-            'button_press_event', self, x, self._mplQtYAxisCoordConversion(y))
+            'button_press_event', self, int(xDisplay), int(yDisplay))
+        # Override axes and data position with the axes
         mouseEvent.inaxes = item.axes
+        mouseEvent.xdata, mouseEvent.ydata = self.pixelToData(
+            x, y, axis='left' if item.axes is self.ax else 'right')
         picked, info = item.contains(mouseEvent)
 
         if not picked:
