@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
+# Copyright (c) 2017-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,7 @@ try:
 except ImportError:  # Python2 support
     import collections as abc
 
-from .core import (Item, AlphaMixIn, BaselineMixIn, ColorMixIn, FillMixIn,
+from .core import (DataItem, AlphaMixIn, BaselineMixIn, ColorMixIn, FillMixIn,
                    LineMixIn, YAxisMixIn, ItemChangedType)
 
 _logger = logging.getLogger(__name__)
@@ -60,17 +60,17 @@ def _computeEdges(x, histogramType):
     """
     # for now we consider that the spaces between xs are constant
     edges = x.copy()
-    if histogramType is 'left':
+    if histogramType == 'left':
         width = 1
         if len(x) > 1:
             width = x[1] - x[0]
         edges = numpy.append(x[0] - width, edges)
-    if histogramType is 'center':
+    if histogramType == 'center':
         edges = _computeEdges(edges, 'right')
         widths = (edges[1:] - edges[0:-1]) / 2.0
         widths = numpy.append(widths, widths[-1])
         edges = edges - widths
-    if histogramType is 'right':
+    if histogramType == 'right':
         width = 1
         if len(x) > 1:
             width = x[-1] - x[-2]
@@ -100,7 +100,7 @@ def _getHistogramCurve(histogram, edges):
 
 
 # TODO: Yerror, test log scale
-class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
+class Histogram(DataItem, AlphaMixIn, ColorMixIn, FillMixIn,
                 LineMixIn, YAxisMixIn, BaselineMixIn):
     """Description of an histogram"""
 
@@ -119,7 +119,7 @@ class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
     _DEFAULT_BASELINE = None
 
     def __init__(self):
-        Item.__init__(self)
+        DataItem.__init__(self)
         AlphaMixIn.__init__(self)
         BaselineMixIn.__init__(self)
         ColorMixIn.__init__(self)
@@ -157,8 +157,8 @@ class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
                 (x <= 0) if xPositive else False,
                 (y <= 0) if yPositive else False)
             # Make a copy and replace negative points by NaN
-            x = numpy.array(x, dtype=numpy.float)
-            y = numpy.array(y, dtype=numpy.float)
+            x = numpy.array(x, dtype=numpy.float64)
+            y = numpy.array(y, dtype=numpy.float64)
             x[clipped] = numpy.nan
             y[clipped] = numpy.nan
 
@@ -170,7 +170,6 @@ class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
                                 yaxis=self.getYAxis(),
                                 xerror=None,
                                 yerror=None,
-                                z=self.getZValue(),
                                 fill=self.isFill(),
                                 alpha=self.getAlpha(),
                                 baseline=baseline,
@@ -188,17 +187,17 @@ class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
             yPositive = False
 
         if xPositive or yPositive:
-            values = numpy.array(values, copy=True, dtype=numpy.float)
+            values = numpy.array(values, copy=True, dtype=numpy.float64)
 
             if xPositive:
                 # Replace edges <= 0 by NaN and corresponding values by NaN
                 clipped_edges = (edges <= 0)
-                edges = numpy.array(edges, copy=True, dtype=numpy.float)
+                edges = numpy.array(edges, copy=True, dtype=numpy.float64)
                 edges[clipped_edges] = numpy.nan
                 clipped_values = numpy.logical_or(clipped_edges[:-1],
                                                   clipped_edges[1:])
             else:
-                clipped_values = numpy.zeros_like(values, dtype=numpy.bool)
+                clipped_values = numpy.zeros_like(values, dtype=bool)
 
             if yPositive:
                 # Replace values <= 0 by NaN, do not modify edges
@@ -213,30 +212,19 @@ class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
                     numpy.nanmax(values))
 
         else:  # No log scale on y axis, include 0 in bounds
+            if numpy.all(numpy.isnan(values)):
+                return None
             return (numpy.nanmin(edges),
                     numpy.nanmax(edges),
                     min(0, numpy.nanmin(values)),
                     max(0, numpy.nanmax(values)))
 
-    def setVisible(self, visible):
-        """Set visibility of item.
-
-        :param bool visible: True to display it, False otherwise
-        """
-        visible = bool(visible)
-        # TODO hackish data range implementation
-        if self.isVisible() != visible:
-            plot = self.getPlot()
-            if plot is not None:
-                plot._invalidateDataRange()
-        super(Histogram, self).setVisible(visible)
-
     def getValueData(self, copy=True):
         """The values of the histogram
 
         :param copy: True (Default) to get a copy,
                      False to use internal representation (do not modify!)
-        :returns: The bin edges of the histogram
+        :returns: The values of the histogram
         :rtype: numpy.ndarray
         """
         return numpy.array(self._histogram, copy=copy)
@@ -298,6 +286,7 @@ class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
 
             # Check that bin edges are monotonic
             edgesDiff = numpy.diff(edges)
+            edgesDiff = edgesDiff[numpy.logical_not(numpy.isnan(edgesDiff))]
             assert numpy.all(edgesDiff >= 0) or numpy.all(edgesDiff <= 0)
             # manage baseline
             if (isinstance(baseline, abc.Iterable)):
@@ -312,11 +301,7 @@ class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
             self._alignement = align
             self._setBaseline(baseline)
 
-        if self.isVisible():
-            plot = self.getPlot()
-            if plot is not None:
-                plot._invalidateDataRange()
-
+        self._boundsChanged()
         self._updated(ItemChangedType.DATA)
 
     def getAlignment(self):
@@ -342,11 +327,11 @@ class Histogram(Item, AlphaMixIn, ColorMixIn, FillMixIn,
         """
         # for now we consider that the spaces between xs are constant
         edges = x.copy()
-        if histogramType is 'left':
+        if histogramType == 'left':
             return edges[1:]
-        if histogramType is 'center':
+        if histogramType == 'center':
             edges = (edges[1:] + edges[:-1]) / 2.0
-        if histogramType is 'right':
+        if histogramType == 'right':
             width = 1
             if len(x) > 1:
                 width = x[-1] + x[-2]
