@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
+# Copyright (c) 2017-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -40,9 +40,8 @@ import logging
 import numpy
 
 from ....utils.proxy import docstring
-from .core import (Item, LabelsMixIn, DraggableMixIn, ColormapMixIn,
+from .core import (DataItem, LabelsMixIn, DraggableMixIn, ColormapMixIn,
                    AlphaMixIn, ItemChangedType)
-from ._pick import PickingResult
 
 
 _logger = logging.getLogger(__name__)
@@ -88,15 +87,20 @@ def _convertImageToRgba32(image, copy=True):
         return numpy.array(image, copy=copy)
 
 
-class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
-    """Description of an image"""
+class ImageBase(DataItem, LabelsMixIn, DraggableMixIn, AlphaMixIn):
+    """Description of an image
 
-    def __init__(self):
-        Item.__init__(self)
+    :param numpy.ndarray data: Initial image data
+    """
+
+    def __init__(self, data=None):
+        DataItem.__init__(self)
         LabelsMixIn.__init__(self)
         DraggableMixIn.__init__(self)
         AlphaMixIn.__init__(self)
-        self._data = numpy.zeros((0, 0, 4), dtype=numpy.uint8)
+        if data is None:
+            data = numpy.zeros((0, 0, 4), dtype=numpy.uint8)
+        self._data = data
 
         self._origin = (0., 0.)
         self._scale = (1., 1.)
@@ -108,7 +112,7 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         elif item == 0:
             return self.getData(copy=False)
         elif item == 1:
-            return self.getLegend()
+            return self.getName()
         elif item == 2:
             info = self.getInfo(copy=False)
             return {} if info is None else info
@@ -130,38 +134,6 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         else:
             raise IndexError("Index out of range: %s" % str(item))
 
-    def setVisible(self, visible):
-        """Set visibility of item.
-
-        :param bool visible: True to display it, False otherwise
-        """
-        visible = bool(visible)
-        # TODO hackish data range implementation
-        if self.isVisible() != visible:
-            plot = self.getPlot()
-            if plot is not None:
-                plot._invalidateDataRange()
-        super(ImageBase, self).setVisible(visible)
-
-    @docstring(Item)
-    def pick(self, x, y):
-        if super(ImageBase, self).pick(x, y) is not None:
-            plot = self.getPlot()
-            if plot is None:
-                return None
-
-            dataPos = plot.pixelToData(x, y)
-            if dataPos is None:
-                return None
-
-            origin = self.getOrigin()
-            scale = self.getScale()
-            column = int((dataPos[0] - origin[0]) / float(scale[0]))
-            row = int((dataPos[1] - origin[1]) / float(scale[1]))
-            return PickingResult(self, ([row], [column]))
-
-        return None
-
     def _isPlotLinear(self, plot):
         """Return True if plot only uses linear scale for both of x and y
         axes."""
@@ -209,6 +181,15 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         """
         return numpy.array(self._data, copy=copy)
 
+    def setData(self, data):
+        """Set the image data
+
+        :param numpy.ndarray data:
+        """
+        self._data = data
+        self._boundsChanged()
+        self._updated(ItemChangedType.DATA)
+
     def getRgbaImageData(self, copy=True):
         """Get the displayed RGB(A) image
 
@@ -235,13 +216,7 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
             origin = float(origin), float(origin)
         if origin != self._origin:
             self._origin = origin
-
-            # TODO hackish data range implementation
-            if self.isVisible():
-                plot = self.getPlot()
-                if plot is not None:
-                    plot._invalidateDataRange()
-
+            self._boundsChanged()
             self._updated(ItemChangedType.POSITION)
 
     def getScale(self):
@@ -264,13 +239,7 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
 
         if scale != self._scale:
             self._scale = scale
-
-            # TODO hackish data range implementation
-            if self.isVisible():
-                plot = self.getPlot()
-                if plot is not None:
-                    plot._invalidateDataRange()
-
+            self._boundsChanged()
             self._updated(ItemChangedType.SCALE)
 
 
@@ -278,9 +247,8 @@ class ImageData(ImageBase, ColormapMixIn):
     """Description of a data image with a colormap"""
 
     def __init__(self):
-        ImageBase.__init__(self)
+        ImageBase.__init__(self, numpy.zeros((0, 0), dtype=numpy.float32))
         ColormapMixIn.__init__(self)
-        self._data = numpy.zeros((0, 0), dtype=numpy.float32)
         self._alternativeImage = None
         self.__alpha = None
 
@@ -301,11 +269,16 @@ class ImageData(ImageBase, ColormapMixIn):
         if dataToUse.size == 0:
             return None  # No data to display
 
+        colormap = self.getColormap()
+        if colormap.isAutoscale():
+            # Avoid backend to compute autoscale: use item cache
+            colormap = colormap.copy()
+            colormap.setVRange(*colormap.getColormapRange(self))
+
         return backend.addImage(dataToUse,
                                 origin=self.getOrigin(),
                                 scale=self.getScale(),
-                                z=self.getZValue(),
-                                colormap=self.getColormap(),
+                                colormap=colormap,
                                 alpha=self.getAlpha())
 
     def __getitem__(self, item):
@@ -331,7 +304,7 @@ class ImageData(ImageBase, ColormapMixIn):
         else:
             # Apply colormap, in this case an new array is always returned
             colormap = self.getColormap()
-            image = colormap.applyToData(self.getData(copy=False))
+            image = colormap.applyToData(self)
             alphaImage = self.getAlphaData(copy=False)
             if alphaImage is not None:
                 # Apply transparency
@@ -385,7 +358,7 @@ class ImageData(ImageBase, ColormapMixIn):
             _logger.warning(
                 'Converting complex image to absolute value to plot it.')
             data = numpy.absolute(data)
-        self._data = data
+        self._setColormappedData(data, copy=False)
 
         if alternative is not None:
             alternative = numpy.array(alternative, copy=copy)
@@ -403,20 +376,14 @@ class ImageData(ImageBase, ColormapMixIn):
                 alpha = numpy.clip(alpha, 0., 1.)
         self.__alpha = alpha
 
-        # TODO hackish data range implementation
-        if self.isVisible():
-            plot = self.getPlot()
-            if plot is not None:
-                plot._invalidateDataRange()
-
-        self._updated(ItemChangedType.DATA)
+        super().setData(data)
 
 
 class ImageRgba(ImageBase):
     """Description of an RGB(A) image"""
 
     def __init__(self):
-        ImageBase.__init__(self)
+        ImageBase.__init__(self, numpy.zeros((0, 0, 4), dtype=numpy.uint8))
 
     def _addBackendRenderer(self, backend):
         """Update backend renderer"""
@@ -434,7 +401,6 @@ class ImageRgba(ImageBase):
         return backend.addImage(data,
                                 origin=self.getOrigin(),
                                 scale=self.getScale(),
-                                z=self.getZValue(),
                                 colormap=None,
                                 alpha=self.getAlpha())
 
@@ -455,15 +421,7 @@ class ImageRgba(ImageBase):
         data = numpy.array(data, copy=copy)
         assert data.ndim == 3
         assert data.shape[-1] in (3, 4)
-        self._data = data
-
-        # TODO hackish data range implementation
-        if self.isVisible():
-            plot = self.getPlot()
-            if plot is not None:
-                plot._invalidateDataRange()
-
-        self._updated(ItemChangedType.DATA)
+        super().setData(data)
 
 
 class MaskImageData(ImageData):
@@ -473,3 +431,82 @@ class MaskImageData(ImageData):
     internal silx widgets.
     """
     pass
+
+
+class ImageStack(ImageData):
+    """Item to store a stack of images and to show it in the plot as one
+    of the images of the stack.
+
+    The stack is a 3D array ordered this way: `frame id, y, x`.
+    So the first image of the stack can be reached this way: `stack[0, :, :]`
+    """
+
+    def __init__(self):
+        ImageData.__init__(self)
+        self.__stack = None
+        """A 3D numpy array (or a mimic one, see ListOfImages)"""
+        self.__stackPosition = None
+        """Displayed position in the cube"""
+
+    def setStackData(self, stack, position=None, copy=True):
+        """Set the stack data
+
+        :param stack: A 3D numpy array like
+        :param int position: The position of the displayed image in the stack
+        :param bool copy: True (Default) to get a copy,
+                          False to use internal representation (do not modify!)
+        """
+        if self.__stack is stack:
+            return
+        if copy:
+            stack = numpy.array(stack)
+        assert stack.ndim == 3
+        self.__stack = stack
+        if position is not None:
+            self.__stackPosition = position
+        if self.__stackPosition is None:
+            self.__stackPosition = 0
+        self.__updateDisplayedData()
+
+    def getStackData(self, copy=True):
+        """Get the stored stack array.
+
+        :param bool copy: True (Default) to get a copy,
+                          False to use internal representation (do not modify!)
+        :rtype: A 3D numpy array, or numpy array like
+        """
+        if copy:
+            return numpy.array(self.__stack)
+        else:
+            return self.__stack
+
+    def setStackPosition(self, pos):
+        """Set the displayed position on the stack.
+
+        This function will clamp the stack position according to
+        the real size of the first axis of the stack.
+
+        :param int pos: A position on the first axis of the stack.
+        """
+        if self.__stackPosition == pos:
+            return
+        self.__stackPosition = pos
+        self.__updateDisplayedData()
+
+    def getStackPosition(self):
+        """Get the displayed position of the stack.
+
+        :rtype: int
+        """
+        return self.__stackPosition
+
+    def __updateDisplayedData(self):
+        """Update the displayed frame whenever the stack or the stack
+        position are updated."""
+        if self.__stack is None or self.__stackPosition is None:
+            empty = numpy.array([]).reshape(0, 0)
+            self.setData(empty, copy=False)
+            return
+        size = len(self.__stack)
+        self.__stackPosition = numpy.clip(self.__stackPosition, 0, size)
+        self.setData(self.__stack[self.__stackPosition], copy=False)
