@@ -1,6 +1,6 @@
 # coding: utf-8
 # /*##########################################################################
-# Copyright (C) 2016-2019 European Synchrotron Radiation Facility
+# Copyright (C) 2016-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -31,7 +31,6 @@ __date__ = "17/01/2018"
 
 
 import unittest
-import warnings
 
 import numpy
 
@@ -88,8 +87,7 @@ class TestMinMax(ParametricTestCase):
                 argmax = numpy.where(data == maximum)[0][0]
 
             if min_positive:
-                with warnings.catch_warnings():
-                    warnings.simplefilter('ignore', category=RuntimeWarning)
+                with numpy.errstate(invalid='ignore'):
                     # Ignore invalid value encountered in greater
                     pos_data = filtered_data[filtered_data > 0]
                 if pos_data.size > 0:
