@@ -4,7 +4,7 @@
 #    Project: Sift implementation in Python + OpenCL
 #             https://github.com/silx-kit/silx
 #
-#    Copyright (C) 2013-2018  European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2013-2020  European Synchrotron Radiation Facility, Grenoble, France
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -436,4 +436,4 @@ class ByteOffset(OpenclProcessing):
         :rtype: bytes
         """
         compressed_array = self.encode(data)
-        return compressed_array.get().tostring()
+        return compressed_array.get().tobytes()
